"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaScriptEngine = void 0;
const ern_core_1 = require("ern-core");
const ern_container_gen_1 = require("ern-container-gen");
const glob_1 = __importDefault(require("glob"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
const semver_1 = __importDefault(require("semver"));
// tslint:disable-next-line:no-var-requires
const AdmZip = require('adm-zip');
const PATH_TO_TEMPLATES_DIR = path_1.default.join(__dirname, 'templates');
const PATH_TO_HULL_DIR = path_1.default.join(__dirname, 'hull');
const ERN_CUSTOM_REACT_NATIVE_AAR_PATCH_VERSION = 100;
var JavaScriptEngine;
(function (JavaScriptEngine) {
    JavaScriptEngine[JavaScriptEngine["HERMES"] = 0] = "HERMES";
    JavaScriptEngine[JavaScriptEngine["JSC"] = 1] = "JSC";
})(JavaScriptEngine = exports.JavaScriptEngine || (exports.JavaScriptEngine = {}));
class AndroidGenerator {
    get name() {
        return 'AndroidGenerator';
    }
    get platform() {
        return 'android';
    }
    async generate(config) {
        return (0, ern_container_gen_1.generateContainer)(config, {
            fillContainerHull: this.fillContainerHull.bind(this),
            postBundle: this.postBundle.bind(this),
        });
    }
    async doesDirectoryContainsKotlinSourceFiles(dir) {
        return new Promise((resolve, reject) => {
            (0, glob_1.default)(path_1.default.join(dir, '**/*.kt'), (err, files) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve((files === null || files === void 0 ? void 0 : files.length) > 0);
                }
            });
        });
    }
    async fillContainerHull(config) {
        const copyFromPath = path_1.default.join(PATH_TO_HULL_DIR, '{.*,*}');
        ern_core_1.shell.cp('-R', copyFromPath, config.outDir);
        // https://github.com/npm/npm/issues/1862 npm renames .gitignore to .npmignore causing the generated container to emit the .gitignore file. This solution below helps to bypass it.
        ern_core_1.shell.mv(`${config.outDir}/gitignore`, `${config.outDir}/.gitignore`);
        const reactNativePlugin = lodash_1.default.find(config.plugins, (p) => p.name === 'react-native');
        if (!reactNativePlugin) {
            throw new Error('react-native was not found in plugins list !');
        }
        if (!reactNativePlugin.version) {
            throw new Error('react-native plugin does not have a version !');
        }
        let mustacheView = {
            customFeatures: [],
            customPermissions: [],
            customRepos: [],
            permissions: [],
        };
        (0, ern_core_1.injectReactNativeVersionKeysInObject)(mustacheView, reactNativePlugin.version);
        const electrodeBridgePlugin = lodash_1.default.find(config.plugins, (p) => p.name === 'react-native-electrode-bridge');
        if (electrodeBridgePlugin) {
            mustacheView.hasElectrodeBridgePlugin = true;
        }
        mustacheView.miniApps = await config.composite.getMiniApps();
        mustacheView.jsMainModuleName = config.jsMainModuleName || 'index';
        await ern_core_1.kax
            .task('Preparing Native Dependencies Injection')
            .run(this.buildAndroidPluginsViews(config.plugins, mustacheView));
        await ern_core_1.kax
            .task('Adding Native Dependencies Hooks')
            .run(this.addAndroidPluginHookClasses(config.plugins, config.outDir));
        ern_core_1.kax.task('Setting Android tools and libraries versions').succeed();
        const versions = ern_core_1.android.resolveAndroidVersions({
            reactNativeVersion: reactNativePlugin.version,
            ...config.androidConfig,
        });
        mustacheView = Object.assign(mustacheView, versions);
        const injectPluginsTaskMsg = 'Injecting Native Dependencies';
        const injectPluginsKaxTask = ern_core_1.kax.task(injectPluginsTaskMsg);
        const replacements = [];
        const androidDependencies = {
            annotationProcessor: [],
            files: [],
            raw: [],
            regular: [],
            transitive: [],
        };
        let isKotlinEnabled = false;
        for (const plugin of config.plugins) {
            if (plugin.name === 'react-native') {
                continue;
            }
            let pluginConfig = await ern_core_1.manifest.getPluginConfig(plugin, 'android');
            if (!pluginConfig) {
                ern_core_1.log.warn(`Skipping ${plugin.name} as it does not have an Android configuration`);
                continue;
            }
            injectPluginsKaxTask.text = `${injectPluginsTaskMsg} [${plugin.name}]`;
            let pathToPluginProject;
            const pluginSourcePath = plugin.basePath;
            if (await ern_core_1.utils.isDependencyPathNativeApiImpl(pluginSourcePath)) {
                // For native api implementations, if a 'ern.pluginConfig' object
                // exists in its package.json, replace pluginConfig with this one.
                const pluginPackageJson = await (0, ern_core_1.readPackageJson)(pluginSourcePath);
                if (pluginPackageJson.ern.pluginConfig) {
                    pluginConfig = pluginPackageJson.ern.pluginConfig.android;
                }
                (0, ern_container_gen_1.populateApiImplMustacheView)(pluginSourcePath, mustacheView, true);
            }
            pathToPluginProject = path_1.default.join(pluginSourcePath, pluginConfig.root);
            if (!isKotlinEnabled) {
                isKotlinEnabled = await this.doesDirectoryContainsKotlinSourceFiles(pathToPluginProject);
            }
            ern_core_1.shell.pushd(pathToPluginProject);
            try {
                if (await ern_core_1.utils.isDependencyPathNativeApiImpl(pluginSourcePath)) {
                    // Special handling for native api implementation as we don't
                    // want to copy the API and bridge code (part of native api implementations projects)
                    const relPathToApiImplSource = path_1.default.normalize('lib/src/main/java/com/ern');
                    const absPathToCopyPluginSourceTo = path_1.default.join(config.outDir, 'lib/src/main/java/com');
                    ern_core_1.shell.cp('-R', relPathToApiImplSource, absPathToCopyPluginSourceTo);
                }
                else {
                    const relPathToPluginSource = pluginConfig.moduleName
                        ? path_1.default.join(pluginConfig.moduleName, 'src/main/java')
                        : path_1.default.join('src/main/java');
                    const absPathToCopyPluginSourceTo = path_1.default.join(config.outDir, 'lib/src/main');
                    if (semver_1.default.gte(reactNativePlugin.version, '0.60.0')) {
                        const convertedFiles = this.convertToAndroidX(relPathToPluginSource);
                        if (convertedFiles > 0) {
                            ern_core_1.log.info(`${plugin.name} contains source files with references to the Android Support Library (android.support.*)`);
                            ern_core_1.log.info(`${convertedFiles} files successfully converted to use AndroidX (androidx.*)`);
                        }
                    }
                    ern_core_1.shell.cp('-R', relPathToPluginSource, absPathToCopyPluginSourceTo);
                }
                const { applyPatch, copy, dependencies, features, permissions, replaceInFile, repositories, } = pluginConfig;
                if (copy) {
                    (0, ern_core_1.handleCopyDirective)(pluginSourcePath, config.outDir, copy);
                }
                if (replaceInFile && Array.isArray(replaceInFile)) {
                    for (const r of replaceInFile) {
                        replacements.push(() => {
                            ern_core_1.log.debug(`Performing string replacement on ${r.path}`);
                            const pathToFile = path_1.default.join(config.outDir, r.path);
                            const fileContent = fs_extra_1.default.readFileSync(pathToFile, 'utf8');
                            const patchedFileContent = fileContent.replace(RegExp(r.string, 'g'), r.replaceWith);
                            fs_extra_1.default.writeFileSync(pathToFile, patchedFileContent, {
                                encoding: 'utf8',
                            });
                        });
                    }
                }
                if (applyPatch) {
                    const { patch, root } = applyPatch;
                    if (!patch) {
                        throw new Error('Missing "patch" property in "applyPatch" object');
                    }
                    if (!root) {
                        throw new Error('Missing "root" property in "applyPatch" object');
                    }
                    const [patchFile, rootDir] = [
                        path_1.default.join(pluginConfig.path, patch),
                        path_1.default.join(config.outDir, root),
                    ];
                    await (0, ern_core_1.gitApply)({ patchFile, rootDir });
                }
                if (dependencies) {
                    const transitivePrefix = 'transitive:';
                    const filesPrefix = 'files';
                    const annotationProcessorPrefix = 'annotationProcessor:';
                    for (const dependency of dependencies) {
                        if (dependency.startsWith(transitivePrefix)) {
                            androidDependencies.transitive.push(dependency.replace(transitivePrefix, ''));
                            ern_core_1.log.warn(`Deprecation warning for ${plugin.name} manifest configuration.
${transitivePrefix} dependency prefix has been deprecated and will be removed in a future release.
You should replace "${transitivePrefix}:${dependency}" with "implementation ('${dependency}') { transitive = true }"`);
                        }
                        else if (dependency.startsWith(filesPrefix)) {
                            androidDependencies.files.push(dependency);
                            ern_core_1.log.warn(`Deprecation warning for ${plugin.name} manifest configuration.
${filesPrefix} dependency prefix has been deprecated and will be removed in a future release.
You should replace "${dependency}" with "implementation ${dependency}"`);
                        }
                        else if (dependency.startsWith(annotationProcessorPrefix)) {
                            androidDependencies.annotationProcessor.push(dependency.replace(annotationProcessorPrefix, ''));
                            ern_core_1.log.warn(`Deprecation warning for ${plugin.name} manifest configuration.
${annotationProcessorPrefix} dependency prefix has been deprecated and will be removed in a future release.
You should replace "${annotationProcessorPrefix}:${dependency}" with "annotationProcessor '${dependency}'"`);
                        }
                        else if (/^[^:\s'(]+:[^:]+:[^:]+$/.test(dependency)) {
                            androidDependencies.regular.push(dependency);
                        }
                        else {
                            androidDependencies.raw.push(dependency);
                        }
                    }
                }
                if (repositories) {
                    mustacheView.customRepos.push(...repositories);
                }
                if (permissions) {
                    mustacheView.customPermissions.push(...permissions);
                }
                if (features) {
                    mustacheView.customFeatures.push(...features);
                }
            }
            finally {
                ern_core_1.shell.popd();
            }
        }
        const resPath = path_1.default.join(config.outDir, 'lib/src/main/res');
        const resSrcDirs = fs_extra_1.default
            .readdirSync(resPath)
            .filter((f) => fs_extra_1.default.statSync(path_1.default.join(resPath, f)).isDirectory())
            .map((d) => `'src/main/res/${d}'`)
            .join(',');
        mustacheView.resSrcDirs = resSrcDirs;
        mustacheView.isKotlinEnabled = isKotlinEnabled;
        // Dedupe repositories and permissions
        mustacheView.customRepos = lodash_1.default.uniq(mustacheView.customRepos);
        mustacheView.customPermissions = lodash_1.default.uniq(mustacheView.customPermissions);
        androidDependencies.raw.push(`api 'com.walmartlabs.ern:react-native:${versions.reactNativeAarVersion}'`);
        if (isKotlinEnabled) {
            androidDependencies.regular.push(`org.jetbrains.kotlin:kotlin-stdlib:${versions.kotlinVersion}`);
        }
        mustacheView.implementations = this.buildImplementationStatements(androidDependencies, versions);
        ern_core_1.log.debug(`Implementation statements to be injected: ${JSON.stringify(mustacheView.implementations)}`);
        if (semver_1.default.patch(versions.reactNativeAarVersion) >=
            ERN_CUSTOM_REACT_NATIVE_AAR_PATCH_VERSION) {
            mustacheView.isCustomReactNativeAar = true;
        }
        injectPluginsKaxTask.succeed(injectPluginsTaskMsg);
        const partialProxy = (name) => {
            return fs_extra_1.default.readFileSync(path_1.default.join(PATH_TO_TEMPLATES_DIR, `${name}.mustache`), 'utf8');
        };
        ern_core_1.log.debug('Patching hull');
        const files = (0, fs_readdir_recursive_1.default)(config.outDir, (f) => !f.endsWith('.jar') && !f.endsWith('.aar') && !f.endsWith('.git'));
        const pathLibSrcMain = path_1.default.normalize('lib/src/main');
        const pathLibSrcMainJniLibs = path_1.default.normalize('lib/src/main/jniLibs');
        const pathLibSrcMainAssets = path_1.default.normalize('lib/src/main/assets');
        const pathLibSrcMainJavaCom = path_1.default.join(pathLibSrcMain, 'java/com');
        const pathLibSrcMainRes = path_1.default.join(pathLibSrcMain, 'res');
        const pathLibSrcMainJavaComWalmartlabsErnContainer = path_1.default.join(pathLibSrcMainJavaCom, 'walmartlabs/ern/container');
        for (const file of files) {
            if ((file.startsWith(pathLibSrcMainJavaCom) &&
                !file.startsWith(pathLibSrcMainJavaComWalmartlabsErnContainer)) ||
                file.startsWith(pathLibSrcMainAssets) ||
                file.startsWith(pathLibSrcMainJniLibs) ||
                file.startsWith(pathLibSrcMainRes)) {
                // We don't want to Mustache process library files. It can lead to bad things
                // We also don't want to process assets files ...
                // We just want to process container specific code (which contains mustache templates)
                continue;
            }
            ern_core_1.log.debug(`Mustaching ${file}`);
            const pathToFile = path_1.default.join(config.outDir, file);
            await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(pathToFile, mustacheView, pathToFile, partialProxy);
        }
        ern_core_1.log.debug('Creating miniapp activities');
        const compositeMiniApps = await config.composite.getMiniApps();
        for (const miniApp of compositeMiniApps) {
            const activityFileName = `${miniApp.pascalCaseName}Activity.java`;
            ern_core_1.log.debug(`Creating ${activityFileName}`);
            const pathToMiniAppActivityMustacheTemplate = path_1.default.join(PATH_TO_TEMPLATES_DIR, 'MiniAppActivity.mustache');
            const pathToOutputActivityFile = path_1.default.join(config.outDir, pathLibSrcMainJavaComWalmartlabsErnContainer, 'miniapps', activityFileName);
            await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(pathToMiniAppActivityMustacheTemplate, miniApp, pathToOutputActivityFile, partialProxy);
        }
        for (const perform of replacements) {
            perform();
        }
        if (semver_1.default.gte(reactNativePlugin.version, '0.60.0')) {
            this.getJavaScriptEngine(config) === JavaScriptEngine.JSC
                ? await ern_core_1.kax
                    .task('Injecting JavaScript engine [JavaScriptCore]')
                    .run(this.injectJavaScriptCoreEngine(config, reactNativePlugin.version))
                : await ern_core_1.kax
                    .task('Injecting JavaScript engine [Hermes]')
                    .run(this.injectHermesEngine(config, reactNativePlugin.version));
        }
    }
    async postBundle(config, bundle, reactNativeVersion) {
        if (this.getJavaScriptEngine(config) === JavaScriptEngine.HERMES) {
            const hermesVersion = config.androidConfig.hermesVersion ||
                ern_core_1.android.getDefaultHermesVersion(reactNativeVersion);
            const hermesCli = await ern_core_1.kax
                .task(`Installing hermes-engine@${hermesVersion}`)
                .run(ern_core_1.HermesCli.fromVersion(hermesVersion));
            await ern_core_1.kax.task('Compiling JS bundle to Hermes bytecode').run(hermesCli.compileReleaseBundle({
                bundleSourceMapPath: bundle.sourceMapPath,
                compositePath: config.composite.path,
                jsBundlePath: bundle.bundlePath,
            }));
            bundle.isHermesBundle = true;
        }
    }
    getJavaScriptEngine(config) {
        return config.androidConfig
            ? config.androidConfig.jsEngine === 'jsc'
                ? JavaScriptEngine.JSC
                : config.androidConfig.jsEngine === 'hermes'
                    ? JavaScriptEngine.HERMES
                    : JavaScriptEngine.JSC
            : JavaScriptEngine.JSC;
    }
    /**
     * Starting with React Native 0.60.0, JavaScriptCore engine is distributed
     * separately from React Native and comes in two variants :
     * 'android-jsc' and 'android-jsc-intl'.
     *
     * More details@ https://github.com/react-native-community/jsc-android-buildscript
     *
     * Prior to React Native 0.60.0, the 'libjsc.so' native library files were
     * shipped inside React Native AAR itself. This is not the case anymore.
     * The 'libjsc.so' files are now distributed in the 'jsc-android' npm package,
     * inside an aar library.
     *
     * This function retrieve the 'jsc-android' package and unzip the AAR
     * matching the desired JSC variant ('android-jsc' or 'android-jsc-intl').
     * It then copy the 'libjsc.so' files to the 'jniLibs' directory of the
     * Container. This way, the JSC engine is shipped within the Container and
     * applications won't crash at runtime when trying to load this library.
     */
    async injectJavaScriptCoreEngine(config, reactNativeVersion) {
        var _a, _b, _c, _d;
        let jscVersion = (_b = (_a = config === null || config === void 0 ? void 0 : config.androidConfig) === null || _a === void 0 ? void 0 : _a.jscVersion) !== null && _b !== void 0 ? _b : ern_core_1.android.getDefaultJSCVersion(reactNativeVersion);
        if (/^\d+$/.test(jscVersion)) {
            // For backward compatibility, to avoid breaking clients
            // that are already providing a version through config that
            // only specifies major excluding minor/patch
            jscVersion = `${jscVersion}.0.0`;
        }
        const jscVariant = (_d = (_c = config === null || config === void 0 ? void 0 : config.androidConfig) === null || _c === void 0 ? void 0 : _c.jscVariant) !== null && _d !== void 0 ? _d : ern_core_1.android.DEFAULT_JSC_VARIANT;
        const workingDir = (0, ern_core_1.createTmpDir)();
        try {
            ern_core_1.shell.pushd(workingDir);
            await ern_core_1.yarn.init();
            await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString(`jsc-android@${jscVersion}`));
            const versionMajor = semver_1.default.major(semver_1.default.coerce(jscVersion).version);
            const jscVersionPath = path_1.default.resolve(`./node_modules/jsc-android/dist/org/webkit/${jscVariant}/r${versionMajor}`);
            const jscAARPath = path_1.default.join(jscVersionPath, `${jscVariant}-r${versionMajor}.aar`);
            return new Promise((resolve) => {
                const unzipOutDir = (0, ern_core_1.createTmpDir)();
                const zip = new AdmZip(jscAARPath);
                zip.extractAllTo(unzipOutDir);
                const unzippedJniPath = path_1.default.join(unzipOutDir, 'jni');
                const containerJniLibsPath = path_1.default.join(config.outDir, 'lib/src/main/jniLibs');
                ern_core_1.shell.cp('-Rf', unzippedJniPath, containerJniLibsPath);
                resolve();
            });
        }
        finally {
            ern_core_1.shell.popd();
        }
    }
    /**
     * Inject hermes engine into the Container
     * Done in a similar way as injectJavaScriptCoreEngine method
     */
    async injectHermesEngine(config, reactNativeVersion) {
        var _a, _b;
        const hermesVersion = (_b = (_a = config === null || config === void 0 ? void 0 : config.androidConfig) === null || _a === void 0 ? void 0 : _a.hermesVersion) !== null && _b !== void 0 ? _b : ern_core_1.android.getDefaultHermesVersion(reactNativeVersion);
        const workingDir = (0, ern_core_1.createTmpDir)();
        try {
            ern_core_1.shell.pushd(workingDir);
            await ern_core_1.yarn.init();
            await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString(`hermes-engine@${hermesVersion}`));
            const hermesAarPath = path_1.default.resolve(`./node_modules/hermes-engine/android/hermes-release.aar`);
            return new Promise((resolve) => {
                const unzipOutDir = (0, ern_core_1.createTmpDir)();
                const zip = new AdmZip(hermesAarPath);
                zip.extractAllTo(unzipOutDir);
                const unzippedJniPath = path_1.default.join(unzipOutDir, 'jni');
                const containerJniLibsPath = path_1.default.join(config.outDir, 'lib/src/main/jniLibs');
                ern_core_1.shell.cp('-Rf', unzippedJniPath, containerJniLibsPath);
                resolve();
            });
        }
        finally {
            ern_core_1.shell.popd();
        }
    }
    buildImplementationStatements(dependencies, androidVersions) {
        const result = [];
        // Replace versions of support libraries with set version
        dependencies.regular = dependencies.regular.map((d) => d.startsWith('androidx.appcompat:')
            ? `${d.slice(0, d.lastIndexOf(':'))}:${androidVersions.androidxAppcompactVersion}`
            : d);
        // Dedupe dependencies with same version
        dependencies.regular = lodash_1.default.uniq(dependencies.regular);
        dependencies.files = lodash_1.default.uniq(dependencies.files);
        dependencies.raw = lodash_1.default.uniq(dependencies.raw);
        dependencies.transitive = lodash_1.default.uniq(dependencies.transitive);
        dependencies.annotationProcessor = lodash_1.default.uniq(dependencies.annotationProcessor);
        // Use highest versions for regular and transitive
        // dependencies with multiple versions
        const g = lodash_1.default.groupBy(dependencies.regular, (x) => x.match(/^[^:]+:[^:]+/)[0]);
        dependencies.regular = Object.keys(g).map((x) => this.highestVersion(g[x]));
        const h = lodash_1.default.groupBy(dependencies.transitive, (x) => x.match(/^[^:]+:[^:]+/)[0]);
        dependencies.transitive = Object.keys(h).map((x) => this.highestVersion(h[x]));
        // Add dependencies to result
        dependencies.regular.forEach((d) => result.push(`implementation '${d}'`));
        dependencies.files.forEach((d) => result.push(`implementation ${d}`));
        dependencies.raw.forEach((d) => {
            result.push(d);
        });
        dependencies.transitive.forEach((d) => result.push(`implementation ('${d}') { transitive = true }`));
        dependencies.annotationProcessor.forEach((d) => result.push(`annotationProcessor '${d}'`));
        return result;
    }
    highestVersion(d) {
        if (d.length === 1) {
            return d[0];
        }
        const name = d[0].match(/^[^:]+:[^:]+/)[0];
        const version = d
            .map((x) => x.match(/^[^:]+:[^:]+:(.+)/)[1])
            // Trick to make highest version lookup as easy
            // as peforming a lexical sort
            .map((x) => x.replace('+', '999999'))
            .sort()
            .map((x) => x.replace('999999', '+'))
            .pop();
        return `${name}:${version}`;
    }
    async addAndroidPluginHookClasses(plugins, outDir) {
        var _a;
        const rnVersion = (_a = plugins.find((p) => p.name === 'react-native')) === null || _a === void 0 ? void 0 : _a.version;
        for (const plugin of plugins) {
            if (plugin.name === 'react-native') {
                continue;
            }
            const pluginConfig = await ern_core_1.manifest.getPluginConfig(plugin, 'android');
            if (!pluginConfig) {
                ern_core_1.log.warn(`Skipping ${plugin.name} as it does not have an Android configuration`);
                continue;
            }
            const androidPluginHook = pluginConfig.pluginHook;
            if (androidPluginHook) {
                ern_core_1.log.debug(`Adding ${androidPluginHook.name}.java`);
                if (!pluginConfig.path) {
                    throw new Error('No plugin config path was set. Cannot proceed.');
                }
                const pathToPluginConfigHook = path_1.default.join(pluginConfig.path, `${androidPluginHook.name}.java`);
                const pathToCopyPluginConfigHookTo = path_1.default.join(outDir, 'lib/src/main/java/com/walmartlabs/ern/container/plugins');
                ern_core_1.shell.cp(pathToPluginConfigHook, pathToCopyPluginConfigHookTo);
                if (semver_1.default.gte(rnVersion, '0.60.0')) {
                    const filesConverted = this.convertToAndroidX(pathToCopyPluginConfigHookTo);
                    if (filesConverted > 0) {
                        ern_core_1.log.info(`${plugin.name} contains source files with references to the Android Support Library (android.support.*)`);
                        ern_core_1.log.info(`${filesConverted} files successfully converted to use AndroidX (androidx.*)`);
                    }
                }
            }
        }
    }
    /**
     * Convert files in a directory from support library to AndroidX
     * eg: import android.support.annotation.NonNull => import androidx.annotation.NonNull
     */
    convertToAndroidX(dir) {
        const filesWithSupportLib = [];
        ern_core_1.shell.pushd(dir);
        ern_core_1.shell
            .ls('-R', '.')
            .filter((file) => file.match(/\.(java|kt)$/))
            .forEach((file) => {
            if (ern_core_1.shell.grep('android.support', file).trim().length !== 0) {
                filesWithSupportLib.push(file);
            }
        });
        filesWithSupportLib.forEach((file) => {
            ern_core_1.shell.sed('-i', 'android.support', 'androidx', file);
        });
        ern_core_1.shell.popd();
        return filesWithSupportLib.length;
    }
    async buildAndroidPluginsViews(plugins, mustacheView) {
        mustacheView.plugins = await (0, ern_container_gen_1.generatePluginsMustacheViews)(plugins, 'android');
        const reactNativeCodePushPlugin = lodash_1.default.find(plugins, (p) => p.name === 'react-native-code-push');
        if (reactNativeCodePushPlugin) {
            mustacheView.isCodePushPluginIncluded = true;
        }
    }
}
exports.default = AndroidGenerator;
//# sourceMappingURL=AndroidGenerator.js.map